{modified from Mesmic 2125 Demo by Jon Williams 
   No rights reserved.  You are free to use this for any purpose.
   rjo

   The purpose of this code was to allow me to study the temperature dependence of the Memsic unit
   What I found was that nearly all of the accumulating error resulted from variability in the
   wave period.  This variability caused the original code to have a variable error by virtue of
   progam logic. This error is only significant if it is allowed to accumulate.  So for angular measurement
   the original code works fine.   But for an accelerometer, there was always an accumulating error that
   seemed to be temperature dependent.  This makes perfect sense to me:)

   By putting additional logic into Jon's object to ensure that I was getting
   the start of the high signal and the lengths of both the on and off perios... and then
   actually computing the ratio of on and off periods,the
   results were nothing short of spectacular:  accumulated error = .096 g... for 123,000 continuous
   measurements over a period of hours... equivaltent to .006 g-sec
   During this time the wave period varied from 801072 to 803120 clocks, using an 80Mhz system clock.

   The additional code reduces the maximum bandwidth to around 50 Hz.  This code is exclusively in Spin
   and takes a fair amount of time writing to the screeen.  The effective measurement rate is almost exactly 16 Hz.  

   It appears that when zero-ing out the initial reading, it may be necessary to have a physical angle that is not "in between" two reliable
   angles. That is, I think it might be necessary to add a small physical
   adjustment to get optimal results.
   .
   I did at my desk by placing a small weight on the leading edge of my board.  This changed the axis deviation
   just enough to eliminate sporatic errors but not to change the offset value.

   To see the effect of temperature on wave period, simply measure the voltage at the analog temperature out pin on
   the memsic unit.  1.250 volts seems to be 25 degrees C.

   I tested the program logic by placing the unit in my car and driving a 3 mile course from and then back to
   my driveway.  In the car,  no physical adjustment was needed.  Everything was stable right out of the box.

   The road trip was to test the hypothesis that if my logic was correct, then the acceleration errors caused by driving up and down steep hills and going around
   corners should all zero out... giving me a final calaculated speed of 0 when I returned back to the starting point
   ... and it worked!!! 
                                               }

                                               

CON

 _clkmode = xtal1 + pll16x
 _xinfreq = 5_000_000
 x_pin = 0
 'y_pin = 26

Var
 long xG,yG,waveperiod,maxlength,minlength,priorperiod,delta,priordelta, d2,tix,tix2,fcnt,mg,mgtot,measurements,counter,mgmax,mgmin
 long t0,t1,t2,mph, cf
Obj
   text : "tv_text"

PUB start
  xG := 100
  text.start(20)
  calibrate(x_pin)
  minlength:=100000000
  maxlength:=0
  priordelta:=0
  priorperiod:=0
  mgtot:=0
  measurements:=0
  counter:=0
  mgmax:=-3000
  mgmin:=3000
  t0:=cnt
  repeat 
    'waitcnt(cnt + clkfreq)
    counter:=counter+1
    text.out(1)
    text.str(string($A,1,$B,1))
    text.str(string("waveperiod  = "))
    text.dec(waveperiod)
    text.str(string("      "))

    waveperiod:= readGforce(x_pin)
    text.out(13)
    
    if maxlength < waveperiod
       maxlength := waveperiod
    if minlength > waveperiod
       minlength := waveperiod
    text.str(string("Max Period Duration (clocks) = "))
    text.dec(maxlength)
    text.out(13)
    text.str(string("Min Period Duration (clocks) = "))
    text.dec(minlength)
    delta:=waveperiod-priorperiod
    priorperiod :=waveperiod
    d2:=delta-priordelta
    priordelta:=delta
    text.out(13)
    text.str(string("d2 = "))
    text.dec(delta)
    text.str(string("    "))
    text.out(13)

    'waitcnt(clkfreq/100 + cnt)
    mg:=((((fcnt-tix2)*1000)/waveperiod)-500)*8          
    text.str(string("mg = "))
    if  mgmax < mg
       mgmax:=mg
    if mgmin > mg
       mgmin:=mg
    mg:=mg-cf
    text.dec(mg)
    text.str(string("    "))
    text.out(13)
    text.str(string("mgtot = "))
    'if ||(mg+cf) > 16
    mgtot:=mgtot+mg
    if counter == 100
      mgtot:=mgtot+0
      counter:=0
    text.dec(mgtot)
    text.str(string("     "))
    text.out(13)
    text.str(string("          MPH = "))
    mph:=(mgtot*60)/46000
    text.dec(mph)
    measurements:=measurements+1
    text.out(13)
    text.str(string("measurements = "))
    text.dec(measurements)
    text.str(string("    "))
    
    text.out(13)
    text.str(string("uncorrected mgmax = "))
    text.dec(mgmax)
    text.str(string("    "))
    
    text.out(13)
    text.str(string("uncorrected mgmin = "))
    text.dec(mgmin)
    text.str(string("    "))

    text.out(13)
    text.str(string("mins = "))
    text.dec(t2/60)
    text.str(string("  secs = "))
    t2:=(measurements*6)/100
    text.dec(t2//60)
    text.str(string("    "))
    text.out(13)
    text.str(string ("cf = "))
    text.dec(cf)

Pub calibrate (pin) | tempx,iii
   tempx := 0
   repeat iii from 1 to 20
     waveperiod:=readgforce(pin)
     mg:=((((fcnt-tix2)*1000)/waveperiod)-500)*8
     if  iii > 10
       tempx:=tempx + mg
  cf:=tempx/10

  
PRI readGforce(pin) | period
 dira[pin]~
 waitpne(|<pin, |< pin, 0)         'pin high...but may be in middle of pulse
 waitpeq(|<pin, |< pin, 0)         'pin low... beginning of low period but not beginning of the signal
 
 waitpne(|<pin, |< pin, 0)         'pin high, beginning of high period and beginning of pulse signal
 tix:=cnt                           'get clock
 waitpeq(|<pin, |< pin, 0)          'pin low...end of high pulse
 tix2:=cnt
 waitpne(|<pin, |< pin, 0)         'pin high end of low period, end of signal wave
 fcnt:=cnt
 period:=(fcnt-tix)
 
 return period